##### R Code for "Expectile and Quantile Regression - David and Goliath?"
library(expectreg)
library(quantreg)

####Calculation of quantiles from expectiles
n <- 499                                  

#Estimate a rich set of expectiles
mp <- 449                   
p <- c(0.00001, 0.00005, 0.0001, 0.0005, 0.001, seq(0 + 1/(mp+1), 1 - 1/(mp+1), length = mp), 0.999, 0.9995, 0.9999, 0.99995, 0.99999)

#Normal distribution
set.seed(126)
#Estimation of expectiles
mu <- expectreg.ls(rnorm(n) ~ 1, expectiles = p)     
mu.norm <- mu$coefficients[[1]]
#Estimation of distribution function
FF.norm <- cdf.qp(mu)$cdf

#Chi-Squared distribution
set.seed(127)
mu <- expectreg.ls(rchisq(n, df = 2, ncp=0) ~ 1, expectiles = p)     
mu.chisq <- mu$coefficients[[1]]
#Estimation of distribution function
FF.chisq <- cdf.qp(mu)$cdf

#t distribution
set.seed(128)
mu <- expectreg.ls(rt(n, df = 3)  ~ 1, expectiles = p)
mu.t <- mu$coefficients[[1]]
#Estimation of distribution function
FF.t <- cdf.qp(mu)$cdf

### Note that a newer version of cdf.qp() offers the possibility to automatically smooth the cdf
par(mar = c(4.3,5.2,1.5,1.5))
plot(FF.norm, p, ylab = expression(hat(h)(alpha)), xlab = expression(alpha), main = '', lty = 'solid', type ='l')
points(pnorm(enorm(p)), p, type ='l', col = 'grey')
points(FF.chisq, p, type = 'l', lty = 'dashed')
points(pchisq(echisq(p[2:(length(p)-2)], df = 2), df = 2), p[2:(length(p)-2)], type ='l', col = 'grey', lty = 'dashed')
points(FF.t, p, type = 'l', lty = 'dotted')
points(pt(et(p[5:(length(p)-5)], df = 3), df = 3), p[5:(length(p)-5)], type ='l', col = 'grey', lty = 'dotted')
legend(0, 1, c("(a) N(0,1)", expression(paste('(b) ',chi^2, '(2)')), '(c) t(3)'),lty = c('solid', 'dashed', 'dotted'))


#Calculation of quantiles and quantiles from expectiles (on which the simulation is based)
mp.quant <- 19
p.quant <- c(0.01, 0.02, seq(0 + 1/(mp.quant+1), 1 - 1/(mp.quant+1), length = mp.quant), 0.98, 0.99)

set.seed(126)
z <- rq(rnorm(n) ~ 1, tau = p.quant)
(quant.quant <- as.vector(z[1][[1]]))
(quant.expect <- as.vector(approx(FF.norm, mu.norm, xout = p.quant, rule = 2)$y))   

plot(quant.expect, quant.quant, ylab = 'Quantiles', xlab = 'Quantiles from Expectiles', lwd = 1.5)
abline(a = 0, b = 1, col = 'grey')


##### Expected shortfall

library(splines)
library(fTrading)
data(EuStockMarkets)

## calculating and plotting the example using the functions defined below
## this may need some minutes
e = eshortfall(EuStockMarkets[,3])
esp = ESpredict(EuStockMarkets[,3],pred=0.1)


### expected shortfall functions
eshortfall <- function(data,theta=c(0.01,0.05,0.95,0.99))
{
  ausduennung = 5
  t = 1:length(data)
  #exp = seq(0.005,0.995,by=0.005)
  exp = c(0.0005,0.001,0.005,seq(0.01,0.99,by=0.01),0.995,0.999,0.9995)
  mm = which(exp == 0.5)

  expreg = expectreg.ls(data ~ rb(t), estimate = 'laws', expectiles = exp, smooth = "fixed", lambda = 0.001)

  res = matrix(0,nrow=length(seq(1,length(data),by=ausduennung)),ncol=length(theta))
  
  for(tt in seq(1,length(data),by=ausduennung))
  {
    cdf = try(cdf.qp(expreg,tt,theta)$quantiles)

      if(class(cdf) == "try-error")
      {
        cdf = NULL
        for(i in 1:length(theta))
        if(theta[i] < 0.5)
          cdf[i] = min(expreg$fitted[tt,])
        else
          cdf[i] = max(expreg$fitted[tt,])
      }
      
    for(i in 1:length(theta))
    {    
      stelle = which.min(abs(expreg$fitted[tt,] - cdf[i]))
      tau = exp[stelle]
   # print(tau)
      if(theta[i] < 0.5)
        korrektur = tau/theta[i]/(1-2*tau)
      else
        korrektur = (1-tau)/(1-theta[i])/(2*tau-1)
      res[(tt-1)/ausduennung+1,i] = (1+korrektur)*expreg$fitted[tt,stelle] - korrektur*expreg$fitted[tt,mm]
    }
   # res = (1+korrektur)*expreg$fitted[tt,stelle] - korrektur*expreg$fitted[tt,rep(mm,length(theta))]
  }

  plot(expreg)

  dev.new()
  #par(mfrow=c(2,1))
  plot(t,data-expreg$fitted[,mm],type="l",ylim = range(data-expreg$fitted[,mm],res-expreg$fitted[seq(1,length(data),by=ausduennung),rep(mm,length(theta))]),main="detrended ES")
  matlines(seq(1,length(data),by=ausduennung),res-expreg$fitted[seq(1,length(data),by=ausduennung),rep(mm,length(theta))],col=rainbow(length(theta)+1)[1:length(theta)], lty=1)
  legend(x="bottomleft", pch=19, cex=1, col=rev(rainbow(length(theta)+1)[1:length(theta)]), legend=rev(theta), bg="white", bty="n")
  
  dev.new()
  plot(t,data,type="l",ylim = range(data,res),main="ES")
  matlines(seq(1,length(data),by=ausduennung),res,col=rainbow(length(theta)+1)[1:length(theta)], lty=1)
  legend(x="topleft", pch=19, cex=1, col=rev(rainbow(length(theta)+1)[1:length(theta)]), legend=rev(theta), bg="white", bty="n")
  
  list(expreg,cbind(t,data),cbind(seq(1,length(data),by=ausduennung),res))
}



ESpredict <- function(data, theta = c(0.01,0.05,0.95,0.99), pred=0.1)
{
  ausduennung = 5
  t = 1:length(data)
  #exp = seq(0.005,0.995,by=0.005)
  exp = c(0.0005,0.001,0.005,seq(0.01,0.99,by=0.01),0.995,0.999,0.9995)
  mm = which(exp == 0.5)

  exp = c(0.0005,0.001,0.005,seq(0.01,0.99,by=0.01),0.995,0.999,0.9995)
  expreg = expectreg.ls(data ~ base3(t, pred = pred), estimate = 'laws', expectiles = exp, smooth = "fixed", lambda=0.3)

  res = matrix(0,nrow=length(seq(1,(1+pred)*length(data),by=ausduennung)),ncol=length(theta))
  
  predata = data.frame(cbind(0,seq(1,(1+pred)*length(data))))
  names(predata) = c("data","t")
  
  pfit = predict(expreg,newdata=predata)
#print(expreg$lambda)
plot(t,data,type="l",xlim=c(1,(1+pred)*length(data)),ylim = range(data,pfit$fitted),main="Expectiles")
matlines(1:((1+pred)*length(data)),pfit$fitted[,seq(1,length(exp),length=15)],col=rainbow(16)[1:15], lty=1)

  orgfit = expreg$fitted
  
  expreg$fitted = pfit$fitted
  expreg$values = pfit$values
  
  for(tt in seq(1,(1+pred)*length(data),by=ausduennung))
  {
    cdf = try(cdf.qp(expreg,tt,theta)$quantiles)

      if(class(cdf) == "try-error")
      {
        cdf = NULL
        for(i in 1:length(theta))
        if(theta[i] < 0.5)
          cdf[i] = min(expreg$fitted[tt,])
        else
          cdf[i] = max(expreg$fitted[tt,])
      }
             
    for(i in 1:length(theta))
    {    
      stelle = which.min(abs(expreg$fitted[tt,] - cdf[i]))
      tau = exp[stelle]
   # print(tau)
      if(theta[i] < 0.5)
        korrektur = tau/theta[i]/(1-2*tau)
      else
        korrektur = (1-tau)/(1-theta[i])/(2*tau-1)
      res[(tt-1)/ausduennung+1,i] = (1+korrektur)*expreg$fitted[tt,stelle] - korrektur*expreg$fitted[tt,mm]
    }
   # res = (1+korrektur)*expreg$fitted[tt,stelle] - korrektur*expreg$fitted[tt,rep(mm,length(theta))]
  }

  #plot(expreg)

  dev.new()
  #par(mfrow=c(2,1))
  plot(t,data-orgfit[,mm],type="l",xlim=c(1,(1+pred)*length(data)),ylim = range(data-orgfit[,mm],res-expreg$fitted[seq(1,(1+pred)*length(data),by=ausduennung),rep(mm,length(theta))]),main="detrended ES")
  matlines(seq(1,(1+pred)*length(data),by=ausduennung),res-expreg$fitted[seq(1,(1+pred)*length(data),by=ausduennung),rep(mm,length(theta))],col=rainbow(length(theta)+1)[1:length(theta)], lty=1)
  legend(x="bottomleft", pch=19, cex=1, col=rev(rainbow(length(theta)+1)[1:length(theta)]), legend=rev(theta), bg="white", bty="n")
  
  dev.new()
  plot(t,data,type="l",xlim=c(1,(1+pred)*length(data)),ylim = range(data,res),main="ES")
  matlines(seq(1,(1+pred)*length(data),by=ausduennung),res,col=rainbow(length(theta)+1)[1:length(theta)], lty=1)
  legend(x="topleft", pch=19, cex=1, col=rev(rainbow(length(theta)+1)[1:length(theta)]), legend=rev(theta), bg="white", bty="n")
  
  res
}



base3 <- function(x,type=c("pspline"),B=NA,P=NA,bnd=NA,center=TRUE,by=NA,pred=.1)
{
  type = match.arg(type)
  Zspathelp = NA
  phi = NA

  xname = deparse(as.list(match.call())$x)


  if(type == "pspline")
  {
    B.deg = 2
    B.size = 20
    diff.size = 1
    
    x0 <- min(x)-0.001
    x1 <- max(x)+0.001 + pred*(max(x)-min(x))
    dx = (x1 - x0)/(B.size-1)
    B = splineDesign(knots=seq(x0-dx*B.deg,x1+dx*B.deg,by=dx),x=x,ord=B.deg+1)

    P <- diag(dim(B)[2])
    for(i in 1:diff.size)
      P <- diff(P)

    if(center)
    {
      # design matrix of fixed effects
      tildeU <- matrix(0,dim(B)[2],diff.size)

      for(i in 1:diff.size)
        tildeU[,i] <- (1:(dim(B)[2]))^(i-1)

      # design matrix of random effects
      tildeZ <- t(P)%*%solve(P%*%t(P))

      # multiply with the original design matrix
      U <- B%*%tildeU
      Z <- B%*%tildeZ
    
      B = cbind(U[,-1],Z)
      P <- diag(c(rep(0,ncol(U)-1), rep(1,ncol(Z))))
    }
  }

constraint = matrix(0,nrow=2,ncol=ncol(P))

  rb = list("B"=B,"P"=P,"x"=x,"type"=type,"bnd"=bnd,"Zspathelp"=Zspathelp,"phi"=phi,"center"=center,"by"=by,"xname"=xname,"pred"=pred,"constraint"=constraint) 
  class(rb) = c("regbase3")
  
  rb
}



predict.regbase3 <- function(object,newdata=NULL,...)
{
  type = object$type
  bnd = object$bnd
  Zspathelp = object$Zspathelp
  phi = object$phi
  center = object$center
  x = object$x
  pred = object$pred
  
  if(is.null(newdata))
    newdata = x
  else if(length(which(names(newdata) == object$xname)) > 0)
    newdata = newdata[,which(names(newdata) == object$xname)]
  else
    stop("Names of newdata not consistent with original.")
  
  if(type == "pspline")
  {
    B.deg = 2
    B.size = 20
    diff.size = 1
    
    x0 <- min(x)-0.001
    x1 <- max(x)+0.001 + pred*(max(x)-min(x))
    dx = (x1 - x0)/(B.size-1)
    B = splineDesign(knots=seq(x0-dx*B.deg,x1+dx*B.deg,by=dx),x=newdata,ord=B.deg+1,outer.ok=TRUE)

    P <- diag(dim(B)[2])
    #for(i in 1:diff.size)
      P <- diff(P,diff = diff.size)
    if(center)
    {
      # design matrix of fixed effects
      tildeU <- matrix(0,dim(B)[2],diff.size)

      for(i in 1:diff.size)
        tildeU[,i] <- (1:(dim(B)[2]))^(i-1)

      # design matrix of random effects
      tildeZ <- t(P)%*%solve(P%*%t(P))

      # multiply with the original design matrix
      U <- B%*%tildeU
      Z <- B%*%tildeZ
    
      B = cbind(U[,-1],Z)
      P <- diag(c(rep(0,ncol(U)-1), rep(1,ncol(Z))))
    }
  }
  
  if (any(!is.na(object$by))) 
        B = B * object$by
  
  B
}

